// COPYRIGHT 


/*****************************************
 INCLUDES
*******************************************/


#include "kppInterfaceWalk/kppCommandLoadHRP2.h"

#include "KineoController/kppSetPropertyCommand.h"
#include "KineoModel/kppComponentParameter.h"
#include "KineoModel/kppValue.h"
#include "KineoModel/kppComponentClassFilter.h"
#include "KineoModel/kppComponent.h"
#include "KineoModel/kppSolidComponent.h"

#include "hppciOpenHrp.h"

#include "hppPolyhedron.h"

#include "kppInterfaceWalkOpenHRPconfig.h"

#include "hrp2_14/hrp2_14.h"

/*****************************************
 DEFINES
*******************************************/


// flag for specific obstacle loading
// #define CASA_OBSTACLE

/*****************************************
 METHODS
*******************************************/



// ==========================================================================

CkppCommandLoadHRP2::CkppCommandLoadHRP2(CkppInterfaceWalk *kpp)
{
  attKpp = kpp;
}


// ==========================================================================

CkppCommandLoadHRP2::CkppCommandLoadHRP2(const CkppCommandLoadHRP2& i_command) :
  CkppCommand(i_command)
{
  attKpp = i_command.attKpp;
}


// ==========================================================================

CkppCommandLoadHRP2::~CkppCommandLoadHRP2()
{
  
}


// ==========================================================================

CkppCommandLoadHRP2ShPtr CkppCommandLoadHRP2::create(CkppInterfaceWalk *kpp)
{
  CkppCommandLoadHRP2*  ptr = new CkppCommandLoadHRP2(kpp);
  CkppCommandLoadHRP2ShPtr shPtr(ptr);
  
  if(KD_OK != ptr->init(shPtr))
    {
      shPtr.reset();
    }

  return shPtr;
}

// ==========================================================================

CkppCommandLoadHRP2ShPtr CkppCommandLoadHRP2::createCopy(const CkppCommandLoadHRP2ConstShPtr& i_command)
{
  CkppCommandLoadHRP2*  ptr = new CkppCommandLoadHRP2(*i_command);
  CkppCommandLoadHRP2ShPtr shPtr(ptr);
  
  if(KD_OK != ptr->init(shPtr))
    {
      shPtr.reset();
    }
  
  return shPtr;
}


// ==========================================================================

ktStatus CkppCommandLoadHRP2::init(const CkppCommandLoadHRP2WkPtr& i_weakPtr)
{
  ktStatus success = CkppCommand::init(i_weakPtr);
  
  if(KD_OK == success)
	{
	  m_weakPtr = i_weakPtr;
	}

  return success;
}


// ==========================================================================


CkppCommandShPtr CkppCommandLoadHRP2::clone() const
{
  return CkppCommandLoadHRP2::createCopy(m_weakPtr.lock());
}


// ==========================================================================

bool CkppCommandLoadHRP2::isUndoable() const
{
  return true;
}


// ==========================================================================

unsigned int CkppCommandLoadHRP2::countParameters() const
{
  return PARAMETER_COUNT;
}


// ==========================================================================

CkppParameterConstShPtr CkppCommandLoadHRP2::parameter(unsigned int i_rank) const
{
 
  CkppParameterShPtr result;
  CkppComponentShPtr nullComponent;

  switch(i_rank)
    {   
    default:
      KPP_ASSERT( false );
    }
  
  return result;
}


// ==========================================================================

ktStatus CkppCommandLoadHRP2::doExecute()
{

  ChppWalkPlanner *hppWalkPlanner = attKpp->hppWalkPlanner();
  if (hppWalkPlanner == NULL )
    {
      cerr << "You need to create a Planner object first." << endl;
      return KD_ERROR;
    }
  
  ChppciOpenHrpClient openHrpClient(hppWalkPlanner);
  if (openHrpClient.loadHrp2Model() != KD_OK)
    {
      return KD_ERROR;
    }

#ifdef CASA_OBSTACLE
  string file_casa = "CASA/envSqrOpenHRP.wrl";
  string name_casa = "CASA";
  CkppKCDPolyhedronShPtr poly_casa;

  if (openHrpClient.loadObstacleModel(file_casa, name_casa, poly_casa) != KD_OK)
    {
      return KD_ERROR;
    }

  hppWalkPlanner->addObstacle(poly_casa);
#endif

  //
  // Set HRP2 in half-sitting position.
  //
  CkppDeviceComponentShPtr hrp2 = hppWalkPlanner->robotIthProblem(0);
  CkwsConfigShPtr halfSittingConfig = CkwsConfig::create(hrp2) ;
  double dInitPos[46] = HALFSITTINGPOSITION_RAD_KINEO ;
  std::vector<double>  halfSittingVector (dInitPos  , dInitPos + sizeof(dInitPos) / sizeof(double) );
  halfSittingConfig->setDofValues( halfSittingVector ) ; 
  hrp2->setCurrentConfig(*halfSittingConfig);

  // Create bounding box robot and place it at beginning of problem vector.
  hppWalkPlanner->initializeProblem();

  // Bound joints
  // Tx
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(0)->isBounded(true);
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(0)->bounds(-0.5,2.5);
  // Ty
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(1)->isBounded(true);
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(1)->bounds(-1.5,1.5);
  //Rz
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(5)->isBounded(true);
  hppWalkPlanner->robotIthProblem(0)->rootJoint()->dof(5)->bounds(-2*M_PI,2*M_PI);

  return KD_OK ;
 
}


// ==========================================================================
