// COPYRIGHT 


/*****************************************
 INCLUDES
*******************************************/


#include "kppInterfaceWalk/kppCommandOutputMotion.h"

#include "KineoController/kppSetPropertyCommand.h"
#include "KineoModel/kppComponentParameter.h"
#include "KineoModel/kppValue.h"
#include "KineoModel/kppComponentClassFilter.h"
#include "KineoModel/kppComponent.h"
#include "KineoModel/kppSolidComponent.h"

#include "KineoModel/kppPathNode.h"
#include "KineoModel/kppPathComponent.h"

#include "KineoController/kppDocument.h"
#include "KineoGUI/kppMainWindowController.h" 

#include "KineoModel/kppModelTree.h"

#include "hppOpenHRP/configSwap.h"

#include <iostream>
#include <fstream>
/*****************************************
 DEFINES
*******************************************/

/*****************************************
 METHODS
*******************************************/



// ==========================================================================

CkppCommandOutputMotion::CkppCommandOutputMotion(CkppInterfaceWalk *kpp)
{
  attKpp = kpp;
}


// ==========================================================================

CkppCommandOutputMotion::CkppCommandOutputMotion(const CkppCommandOutputMotion& i_command) :
  CkppCommand(i_command)
{
  attKpp = i_command.attKpp;
}


// ==========================================================================

CkppCommandOutputMotion::~CkppCommandOutputMotion()
{
  
}


// ==========================================================================

CkppCommandOutputMotionShPtr CkppCommandOutputMotion::create(CkppInterfaceWalk *kpp)
{
  CkppCommandOutputMotion*  ptr = new CkppCommandOutputMotion(kpp);
  CkppCommandOutputMotionShPtr shPtr(ptr);
  
  if(KD_OK != ptr->init(shPtr))
    {
      shPtr.reset();
    }

  return shPtr;
}

// ==========================================================================

CkppCommandOutputMotionShPtr CkppCommandOutputMotion::createCopy(const CkppCommandOutputMotionConstShPtr& i_command)
{
  CkppCommandOutputMotion*  ptr = new CkppCommandOutputMotion(*i_command);
  CkppCommandOutputMotionShPtr shPtr(ptr);
  
  if(KD_OK != ptr->init(shPtr))
    {
      shPtr.reset();
    }
  
  return shPtr;
}


// ==========================================================================

ktStatus CkppCommandOutputMotion::init(const CkppCommandOutputMotionWkPtr& i_weakPtr)
{
  ktStatus success = CkppCommand::init(i_weakPtr);
  
  if(KD_OK == success)
	{
	  m_weakPtr = i_weakPtr;
	}

  return success;
}


// ==========================================================================


CkppCommandShPtr CkppCommandOutputMotion::clone() const
{
  return CkppCommandOutputMotion::createCopy(m_weakPtr.lock());
}


// ==========================================================================

bool CkppCommandOutputMotion::isUndoable() const
{
  return true;
}


// ==========================================================================

unsigned int CkppCommandOutputMotion::countParameters() const
{
  return PARAMETER_COUNT;
}


// ==========================================================================

CkppParameterConstShPtr CkppCommandOutputMotion::parameter(unsigned int i_rank) const
{
 
  CkppParameterShPtr result;
  CkppComponentShPtr nullComponent;

  switch(i_rank) {   
  case PATH:
    result = CkppComponentParameter::create("selected path", CkppComponentClassFilter<CkppPathComponent>());
    break;
  default:
    KPP_ASSERT( false );
  }
  
  return result;
}


// ==========================================================================

ktStatus CkppCommandOutputMotion::doExecute()
{
  ktStatus status;

  string filename("hppWalk");


  vector< CkwsConfigShPtr > 	 outHRP2Path;
  vector< double > outHRP2TimeRef;
  vector< vector< double > > outHRP2ZMPRef;
  vector< vector< double > > outHRP2WaistPosition;

  status = attKpp->hppWalkPlanner()->solution(outHRP2Path, outHRP2TimeRef, outHRP2ZMPRef, outHRP2WaistPosition);

  if(status == KD_ERROR){
    cerr<<"CkppCommandOutputMotion::doExecute(): could not get solution"<<endl;
    return status;
  }

  cout<<"size path:" <<outHRP2Path.size()<<", outHRP2TimeRef: "<<outHRP2TimeRef.size()
      <<", outHRP2ZMPRef: "<<outHRP2ZMPRef.size()<<", outHRP2WaistPosition: "<<outHRP2WaistPosition.size()<<endl;

  ofstream fout_pos((filename+".pos").c_str());
  ofstream fout_zmp((filename+".zmp").c_str());
  ofstream fout_rpy((filename+".rpy").c_str());
  ofstream fout_waistpos((filename+".waistpos").c_str());

  for(unsigned int i=0; i<outHRP2Path.size(); i++){

    fout_pos<<outHRP2TimeRef[i];
    fout_zmp<<outHRP2TimeRef[i];
    fout_rpy<<outHRP2TimeRef[i];
    fout_waistpos<<outHRP2TimeRef[i];


    vector <double> openHRPConfig(40);

    // cout<<"size dof cfg "<<outHRP2Path[i]->size()<<endl;
    toOpenHRPConfig (outHRP2Path[i], openHRPConfig);
    for(unsigned int j=0; j<openHRPConfig.size(); j++){
      fout_pos<<" "<<openHRPConfig[j];
    }
    fout_pos<<endl;

    for(unsigned int j=0; j<outHRP2ZMPRef[i].size(); j++)
      fout_zmp<<" "<<outHRP2ZMPRef[i][j];
    fout_zmp<<endl;
	
    for(unsigned int j=0; j<3; j++){
      fout_rpy<<" "<<outHRP2WaistPosition[i][j+3];
      fout_waistpos<<" "<<outHRP2WaistPosition[i][j];
    }
    fout_rpy<<endl;
    fout_waistpos<<endl;
  }

  fout_pos.close();
  fout_rpy.close();
  fout_zmp.close();
  fout_waistpos.close();

  return KD_OK ;
 
}


// ==========================================================================
